-------------------------------------------------------------------------------
-- Project  ELE8304 : Circuits intégrés à très grande échelle
-------------------------------------------------------------------------------
-- File     shifter.vhd
-- Authors  Titouan Luard <luardtitouan@gmail.com> 
--          Yann Roberge <yann.roberge@polymtl.ca> 
-- Lab      GRM - Polytechnique Montreal
-- Date     2021-10-29
-------------------------------------------------------------------------------
-- Brief    Single-bit half-adder with carry out
------------------------------------------------------------------------
-------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;


library work;
use work.riscv_pkg.all;

entity shifter is
  port (
    i_shamt  : in std_logic_vector(SHAMT_WIDTH-1 downto 0); -- nombre de bits décalés
    i_src1   : in std_logic_vector(XLEN-1 downto 0); -- nombre décalé
    i_arith  : in std_logic; -- 0: décalage logique (bourre des 0) ; 1
    i_opcode : in std_logic_vector(ALUOP_WIDTH-1 downto 0);

    o_sh  : out std_logic_vector(XLEN-1 downto 0)
  );
end entity shifter;

--SRL 000
--SRA 001
--SLL 010
architecture beh of shifter is
begin

shift: 
process (all) is
  variable temp : std_logic_vector(XLEN-1 downto 0);
begin
  case i_opcode is
    when ALUOP_SH_RIGHT =>
      if i_arith = '0' then
        -- Décalage a droite logique 
        o_sh <= i_src1 srl to_integer(unsigned(i_shamt));
      else
        -- Décalage a droite arithmétique 
        if i_src1(XLEN-1) = '0' then
          o_sh <= i_src1 srl to_integer(unsigned(i_shamt));
        else
          --o_sh <= i_src1 sra to_integer(unsigned(i_shamt)); -- NOTE: opérateur "sra" pas implémenté par numeric_std
          temp := i_src1 srl to_integer(unsigned(i_shamt));
          o_sh <= not temp(temp'length - 1 downto temp'length - to_integer(unsigned(i_shamt))) & temp(temp'length - 1 - to_integer(unsigned(i_shamt)) downto 0);
          --o_sh <= not temp(temp'length - 1 downto temp'length - to_integer(unsigned(i_shamt))) & i_src1(temp'length - 1 - to_integer(unsigned(i_shamt)) downto 0);
        end if;
      end if;
    when ALUOP_SH_LEFT =>
      -- Décalage a gauche (logique)
      o_sh <= i_src1 sll to_integer(unsigned(i_shamt));

    when others =>
      -- Pas de décalage
      o_sh <= i_src1;

  end case;
end process shift;


end architecture beh;

